# Description: Shows how to use graphs
# Category:    general
# Classes:     Graph
# Uses:        
# Referenced:  graph.htm

import Orange.network

graph = Orange.network.GraphAsMatrix(4, 0)
graph.objects = ["Age", "Gender", "Height", "Weight"]

graph["Age", "Gender"] = 0.1
graph["Age", "Height"] = 1.2
graph["Gender", "Height"] = 0.3
print graph.getEdges()

print graph[1, 2]
print graph["Gender", "Height"]

graph.objects = {}
graph.objects["Age"] = 0
graph.objects["Gender"] = 1
graph.objects["Height"] = 2

print graph["Age", "Gender"]
try:
    print graph["Gender", "Height"]
except:
    print 'graph["Gender", "Height"] failed'

print graph.getNeighbours("Age")
graph.returnIndices = 1
print graph.getNeighbours("Age")


graph = Orange.network.GraphAsMatrix(5, 0, 3)
print graph[4, 1]
graph[4, 1, 1]=12
print graph[4, 1, 1]
print graph[4, 1]

print graph.edgeExists(4, 1)
print graph.edgeExists(4, 2)
print graph.edgeExists(4, 1, 1)
print graph.edgeExists(4, 1, 2)

e = graph[4, 1]
e[1]
e[2]
e[1] = None
e[2] = 3
print graph.edgeExists(4, 1, 1)
print graph.edgeExists(4, 1, 2)
print e
graph[4, 1]=None
print e



"""
<name>Net Inside View</name>
<description>Orange widget for community detection in networks</description>
<icon>icons/NetworkInsideView.svg</icon>
<contact>Miha Stajdohar (miha.stajdohar(@at@)gmail.com)</contact> 
<priority>6460</priority>
"""

import Orange
import OWGUI

from OWWidget import *

class NxInsideView(Orange.network.NxView):
    """Network Inside View
    
    """

    def __init__(self, nhops):
        Orange.network.NxView.__init__(self)

        self._nhops = nhops
        self._center_node = None

    def init_network(self, graph):
        self._network = graph

        if graph is None:
            return None

        selected = self._nx_explorer.networkCanvas.selected_nodes()
        if selected is None or len(selected) <= 0:
            self._center_node = graph.nodes_iter().next()
        else:
            self._center_node = selected[0]

        nodes = self._get_neighbors()
        return Orange.network.nx.Graph.subgraph(self._network, nodes)

    def update_network(self):
        nodes = self._get_neighbors()
        subnet = Orange.network.nx.Graph.subgraph(self._network, nodes)

        if self._nx_explorer is not None:
            self._nx_explorer.change_graph(subnet)

    def set_nhops(self, nhops):
        self._nhops = nhops

    def node_selection_changed(self):
        selection = self._nx_explorer.networkCanvas.selected_nodes()
        if len(selection) == 1:
            self._center_node = selection[0]
            self.update_network()

    def _get_neighbors(self):
        nodes = set([self._center_node])
        for n in range(self._nhops):
            neighbors = set()
            for node in nodes:
                neighbors.update(self._network.neighbors(node))
            nodes.update(neighbors)
        return nodes

class OWNxInsideView(OWWidget):

    settingsList = ['_nhops']

    def __init__(self, parent=None, signalManager=None):
        OWWidget.__init__(self, parent, signalManager, 'Net Inside View')

        self.inputs = []
        self.outputs = [("Nx View", Orange.network.NxView)]

        self._nhops = 2

        self.loadSettings()

        ib = OWGUI.widgetBox(self.controlArea, "Preferences", orientation="vertical")
        OWGUI.spin(ib, self, "_nhops", 1, 6, 1, label="Number of hops: ", callback=self.update_view)

        self.inside_view = NxInsideView(self._nhops)
        self.send("Nx View", self.inside_view)

    def update_view(self):
        self.inside_view.set_nhops(self._nhops)

        self.inside_view.update_network()



CIRCLE = 0
SQUARE = 1
ROUND_RECT = 2

NOTHING = 0
ZOOMING = 1
SELECT_RECTANGLE = 2
SELECT_POLYGON = 3
MOVE_SELECTION = 100

import numpy

from plot.owplot import *
from plot.owpoint import *
from plot.owtools import *

from Orange import core, data, misc, network, orangeqt, projection
from orngScaleScatterPlotData import *

class NodeItem(orangeqt.NodeItem):
    def __init__(self, index, x=None, y=None, parent=None):
        orangeqt.NodeItem.__init__(self, index, OWPoint.Ellipse, Qt.blue, 5, parent)
        if x is not None:
            self.set_x(x)
        if y is not None:
            self.set_y(y)

class EdgeItem(orangeqt.EdgeItem):
    def __init__(self, u=None, v=None, weight=1, links_index=0, arrows=None, label='', parent=None):
        orangeqt.EdgeItem.__init__(self, u, v, parent)
        self.set_weight(weight)
        self.set_links_index(links_index)
        if arrows is not None:
            self.set_arrows(arrows)

class NetworkCurve(orangeqt.NetworkCurve):
    def __init__(self, parent=None, pen=QPen(Qt.black), xData=None, yData=None):
        orangeqt.NetworkCurve.__init__(self, parent)
        self.name = "Network Curve"

    def layout_fr(self, steps, weighted=False, smooth_cooling=False):
        orangeqt.NetworkCurve.fr(self, steps, weighted, smooth_cooling)

    def set_node_sizes(self, values={}, min_size=0, max_size=0):
        orangeqt.NetworkCurve.set_node_sizes(self, values, min_size, max_size)

    def fragviz_callback(self, a, b, mds, mdsRefresh, graph_components, matrix_components, progress_callback):
        """Refresh the UI when running  MDS on network components."""

        if not self.mdsStep % mdsRefresh:
            rotationOnly = False
            component_props = []
            x_mds = []
            y_mds = []
            phi = [None] * len(graph_components)
            nodes = self.nodes()
            ncomponents = len(graph_components)

            for i in range(ncomponents):

                if len(mds.points) == ncomponents:  # if average linkage before
                    x_avg_mds = mds.points[i][0]
                    y_avg_mds = mds.points[i][1]
                else:                                   # if not average linkage before
                    x = [mds.points[u][0] for u in matrix_components[i]]
                    y = [mds.points[u][1] for u in matrix_components[i]]

                    x_avg_mds = sum(x) / len(x)
                    y_avg_mds = sum(y) / len(y)
                    # compute rotation angle
#                    c = [numpy.linalg.norm(numpy.cross(mds.points[u], \
#                                [nodes[u].x(), nodes[u].y()])) for u in component]
#                    
#                    n = [numpy.vdot([nodes[u].x(), nodes[u].y()], \
#                                    [nodes[u].x(), nodes[u].y()]) for u in component]
#                    phi[i] = sum(c) / sum(n)


                x = [nodes[j].x() for j in graph_components[i]]
                y = [nodes[j].y() for j in graph_components[i]]

                x_avg_graph = sum(x) / len(x)
                y_avg_graph = sum(y) / len(y)

                x_mds.append(x_avg_mds)
                y_mds.append(y_avg_mds)

                component_props.append((x_avg_graph, y_avg_graph, \
                                        x_avg_mds, y_avg_mds, phi))

            for i, component in enumerate(graph_components):
                x_avg_graph, y_avg_graph, x_avg_mds, \
                y_avg_mds, phi = component_props[i]

    #            if phi[i]:  # rotate vertices
    #                #print "rotate", i, phi[i]
    #                r = numpy.array([[numpy.cos(phi[i]), -numpy.sin(phi[i])], [numpy.sin(phi[i]), numpy.cos(phi[i])]])  #rotation matrix
    #                c = [x_avg_graph, y_avg_graph]  # center of mass in FR coordinate system
    #                v = [numpy.dot(numpy.array([self.graph.coors[0][u], self.graph.coors[1][u]]) - c, r) + c for u in component]
    #                self.graph.coors[0][component] = [u[0] for u in v]
    #                self.graph.coors[1][component] = [u[1] for u in v]

                # translate vertices
                if not rotationOnly:
                    self.set_node_coordinates(dict(
                       (j, ((nodes[j].x() - x_avg_graph) + x_avg_mds,
                            (nodes[j].y() - y_avg_graph) + y_avg_mds)) \
                                  for j in component))

            #if self.mdsType == MdsType.exactSimulation:
            #    self.mds.points = [[self.graph.coors[0][i], \
            #                        self.graph.coors[1][i]] \
            #                        for i in range(len(self.graph.coors))]
            #    self.mds.freshD = 0

            self.plot().update_graph_layout()
            qApp.processEvents()

            if progress_callback is not None:
                progress_callback(a, self.mdsStep)

        self.mdsStep += 1
        return 0 if self.stopMDS else 1

    def layout_fragviz(self, steps, distances, graph, progress_callback=None, opt_from_curr=False):
        """Position the network components according to similarities among 
        them.
        
        """

        if distances == None or graph == None or distances.dim != graph.number_of_nodes():
            self.information('invalid or no distance matrix')
            return 1

        p = self.plot()
        edges = self.edges()
        nodes = self.nodes()

        avgLinkage = True
        rotationOnly = False
        minStressDelta = 0
        mdsRefresh = 10#int(steps / 20)

        self.mdsStep = 1
        self.stopMDS = False

        nodes_inds = dict((n, i) for i, n in enumerate(sorted(graph.nodes_iter())))
        inds_nodes = dict((i, n) for i, n in enumerate(sorted(graph.nodes_iter())))

        graph_components = network.nx.algorithms.connected_components(graph)
        matrix_components = [[nodes_inds[n] for n in c] for c in graph_components]

        distances.matrixType = misc.SymMatrix.Symmetric

        # scale net coordinates
        if avgLinkage:
            distances = distances.avgLinkage(matrix_components)

        # if only one component
        if distances.dim == 1:
            return 0

        mds = projection.mds.MDS(distances)
        mds.optimize(10, projection.mds.SgnRelStress, 0)
        rect = self.data_rect()
        w_fr = rect.width()
        h_fr = rect.height()
        d_fr = math.sqrt(w_fr ** 2 + h_fr ** 2)

        x_mds, y_mds = zip(*mds.points)
        w_mds = max(x_mds) - min(x_mds)
        h_mds = max(y_mds) - min(y_mds)
        d_mds = math.sqrt(w_mds ** 2 + h_mds ** 2)

        animate_points = p.animate_points
        p.animate_points = False

        # if only one component
        if d_mds == 0 or d_fr == 0:
            d_mds = 1
            d_fr = 1

        self.set_node_coordinates(dict( (key,(node.x() * d_mds / d_fr, node.y() * d_mds / d_fr)) \
                                   for key, node in nodes.iteritems()))

        p.update_graph_layout()
        qApp.processEvents()

        if opt_from_curr:
            if avgLinkage:
                for u, c in enumerate(graph_components):
                    x = sum([nodes[n].x() for n in c]) / len(c)
                    y = sum([nodes[n].y() for n in c]) / len(c)
                    mds.points[u][0] = x
                    mds.points[u][1] = y
            else:
                for i, u in enumerate(sorted(nodes.iterkeys())):
                    mds.points[i][0] = nodes[u].x()
                    mds.points[i][1] = nodes[u].y()
        else:
            mds.Torgerson()

        mds.optimize(steps, projection.mds.SgnRelStress, minStressDelta,
                     progressCallback=
                         lambda a, b=None, mds=mds, mdsRefresh=mdsRefresh, graph_comp=graph_components,
                                matrix_comp=matrix_components, progress_callback=progress_callback:
                         self.fragviz_callback(a, b, mds, mdsRefresh, graph_comp, matrix_comp, progress_callback))

        self.fragviz_callback(mds.avgStress, 0, mds, mdsRefresh, graph_components, matrix_components, progress_callback)

        if progress_callback is not None:
            progress_callback(mds.avgStress, self.mdsStep)

        p.animate_points = animate_points
        return 0

    def mds_callback(self, a, b, mds, mdsRefresh, progress_callback):
        """Refresh the UI when running  MDS."""

        if not self.mdsStep % mdsRefresh:

            self.set_node_coordinates(dict((n, (mds.points[i][0], \
                                           mds.points[i][1])) for i, n in enumerate(sorted(self.nodes()))))
            self.plot().update_graph_layout()
            qApp.processEvents()

            if progress_callback is not None:
                progress_callback(a, self.mdsStep)

        self.mdsStep += 1
        return 0 if self.stopMDS else 1

    def layout_mds(self, steps, distances, progress_callback=None, opt_from_curr=False):
        """Position the network components according to similarities among 
        them.
        
        """
        nodes = self.nodes()

        if distances == None or distances.dim != len(nodes):
            self.information('invalid or no distance matrix')
            return 1

        p = self.plot()

        minStressDelta = 0
        mdsRefresh = int(steps / 20)

        self.mdsStep = 1
        self.stopMDS = False

        distances.matrixType = core.SymMatrix.Symmetric
        mds = projection.mds.MDS(distances)
        mds.optimize(10, projection.mds.SgnRelStress, 0)
        rect = self.data_rect()
        w_fr = rect.width()
        h_fr = rect.height()
        d_fr = math.sqrt(w_fr ** 2 + h_fr ** 2)

        x_mds, y_mds = zip(*mds.points)
        w_mds = max(x_mds) - min(x_mds)
        h_mds = max(y_mds) - min(y_mds)
        d_mds = math.sqrt(w_mds ** 2 + h_mds ** 2)

        animate_points = p.animate_points
        p.animate_points = False

        self.set_node_coordinates(dict(
           (n, (nodes[n].x() * d_mds / d_fr, nodes[n].y() * d_mds / d_fr)) for n in nodes))

        p.update_graph_layout()
        qApp.processEvents()

        if opt_from_curr:
            for i, u in enumerate(sorted(nodes.iterkeys())):
                mds.points[i][0] = nodes[u].x()
                mds.points[i][1] = nodes[u].y()
        else:
            mds.Torgerson()

        mds.optimize(steps, projection.mds.SgnRelStress, minStressDelta,
                     progressCallback=
                         lambda a,
                                b=None,
                                mds=mds,
                                mdsRefresh=mdsRefresh,
                                progress_callback=progress_callback:
                                    self.mds_callback(a, b, mds, mdsRefresh, progress_callback))

        self.mds_callback(mds.avgStress, 0, mds, mdsRefresh, progress_callback)

        if progress_callback is not None:
            progress_callback(mds.avgStress, self.mdsStep)

        p.animate_points = animate_points
        return 0

#    def move_selected_nodes(self, dx, dy):
#        selected = self.get_selected_nodes()
#        
#        self.coors[selected][0] = self.coors[0][selected] + dx
#        self.coors[1][selected][1] = self.coors[1][selected] + dy
#          
#        self.update_properties()
#        return selected
#        
#    def set_hidden_nodes(self, nodes):
#        for vertex in self.nodes().itervalues():
#            vertex.setVisible(vertex.index() in nodes)
#      
#    def hide_selected_nodes(self):
#        for vertex in self.nodes().itervalues():
#          if vertex.selected:
#            vertex.hide()
#  
#    def hide_unselected_nodes(self):
#        for vertex in self.nodes().itervalues():
#          if not vertex.selected:
#            vertex.hide()
#    
#    def show_all_vertices(self):
#        for vertex in self.nodes().itervalues():
#          vertex.show()



class OWNxCanvas(OWPlot):
    def __init__(self, master, parent=None, name="None"):
        OWPlot.__init__(self, parent, name, axes=[])
        self.master = master
        self.parent = parent
        self.NodeItem = NodeItem
        self.graph = None

        self.circles = []
        self.freezeNeighbours = False
        self.labelsOnMarkedOnly = 0

        self.show_indices = False
        self.show_weights = False
        self.trim_label_words = 0
        self.show_component_distances = False

        self.showComponentAttribute = None
        self.forceVectors = None
        #self.appendToSelection = 1
        self.fontSize = 12

        self.networkCurve = NetworkCurve()
        self.add_custom_curve(self.networkCurve)

        self.minComponentEdgeWidth = 0
        self.maxComponentEdgeWidth = 0
        self.items_matrix = None

        self.items = None
        self.links = None
        self.edge_to_row = None
        self.label_distances = None

        self.node_label_attributes = []
        self.edge_label_attributes = []

        self.axis_margin = 0
        self.title_margin = 0
        self.graph_margin = 1
        self._legend_margin = QRectF(0, 0, 0, 0)

        #self.setFocusPolicy(Qt.StrongFocus)

    def set_hidden_nodes(self, nodes):
        self.networkCurve.set_hidden_nodes(nodes)

    def hide_selected_nodes(self):
      self.networkCurve.hide_selected_nodes()
      self.drawPlotItems()

    def hide_unselected_nodes(self):
      self.networkCurve.hide_unselected_nodes()
      self.drawPlotItems()

    def show_all_vertices(self):
      self.networkCurve.show_all_vertices()
      self.drawPlotItems()

    def selected_nodes(self):
        return [vertex.index() for vertex in self.networkCurve.nodes().itervalues() if vertex.is_selected()]
        #return [p.index() for p in self.selected_points()]

    def not_selected_nodes(self):
        return [vertex.index() for vertex in self.networkCurve.nodes().itervalues() if not vertex.is_selected()]

    def marked_nodes(self):
        return [vertex.index() for vertex in self.networkCurve.nodes().itervalues() if vertex.is_marked()]
        #return [p.index() for p in self.marked_points()]

    def not_marked_nodes(self):
        return [vertex.index() for vertex in self.networkCurve.nodes().itervalues() if not vertex.is_marked()]

    def get_neighbors_upto(self, ndx, dist):
        newNeighbours = neighbours = set([ndx])
        for d in range(dist):
            tNewNeighbours = set()
            for v in newNeighbours:
                tNewNeighbours |= set(self.graph.neighbors(v))
            newNeighbours = tNewNeighbours - neighbours
            neighbours |= newNeighbours
        return neighbours

    def mark_on_selection_changed(self):
        toMark = set()
        for ndx in self.selected_nodes():
            toMark |= self.get_neighbors_upto(ndx, self.mark_neighbors)

        self.networkCurve.clear_node_marks()
        self.networkCurve.set_node_marks(dict((i, True) for i in toMark))

    def mark_on_focus_changed(self, node):
        self.networkCurve.clear_node_marks()

        if node is not None:
            toMark = set(self.get_neighbors_upto(node.index(), self.mark_neighbors))
            self.networkCurve.set_node_marks(dict((i, True) for i in toMark))

    def drawComponentKeywords(self):
        self.clear_markers()
        if not hasattr(self, "showComponentAttribute") or self.showComponentAttribute is None or self.graph is None or self.items is None:
            return

        if str(self.showComponentAttribute) not in self.items.domain:
            self.showComponentAttribute = None
            return

        components = network.nx.algorithms.components.connected_components(self.graph)
        nodes = self.networkCurve.nodes()

        for c in components:
            if len(c) == 0:
                continue

            x1 = sum(nodes[n].x() for n in c) / len(c)
            y1 = sum(nodes[n].y() for n in c) / len(c)
            lbl = str(self.items[c[0]][str(self.showComponentAttribute)])

            self.add_marker(lbl, x1, y1, alignment=Qt.AlignCenter, size=self.fontSize)

    def getColorIndeces(self, table, attribute, palette):
        colorIndices = {}
        colorIndex = None
        minValue = None
        maxValue = None

        if attribute[0] != "(" or attribute[ -1] != ")":
            i = 0
            for var in table.domain.variables:
                if var.name == attribute:
                    colorIndex = i
                    if var.varType == core.VarTypes.Discrete:
                        colorIndices = getVariableValueIndices(var, colorIndex)

                i += 1
            metas = table.domain.getmetas()
            for i, var in metas.iteritems():
                if var.name == attribute:
                    colorIndex = i
                    if var.varType == core.VarTypes.Discrete:
                        colorIndices = getVariableValueIndices(var, colorIndex)

        colorIndices['?'] = len(colorIndices)
        palette.setNumberOfColors(len(colorIndices))

        if colorIndex != None and table.domain[colorIndex].varType == core.VarTypes.Continuous:
            minValue = float(min([x[colorIndex].value for x in table if x[colorIndex].value != "?"] or [0.0]))
            maxValue = float(max([x[colorIndex].value for x in table if x[colorIndex].value != "?"] or [0.0]))

        return colorIndices, colorIndex, minValue, maxValue

    def set_node_colors(self, attribute, nodes=None):
        if self.graph is None:
            return

        colorIndices, colorIndex, minValue, maxValue = self.getColorIndeces(self.items, attribute, self.discPalette)
        colors = {}

        if nodes is None:
            nodes = self.graph.nodes()

        if colorIndex is not None and self.items.domain[colorIndex].varType == core.VarTypes.Continuous and minValue == maxValue:
            colors.update((node, self.discPalette[0]) for node in nodes)

        elif colorIndex is not None and self.items.domain[colorIndex].varType == core.VarTypes.Continuous:
            colors.update((v, self.contPalette[(float(self.items[v][colorIndex].value) - minValue) / (maxValue - minValue)])
                          if str(self.items[v][colorIndex].value) != '?' else
                          (v, self.discPalette[0]) for v in nodes)

        elif colorIndex is not None and self.items.domain[colorIndex].varType == core.VarTypes.Discrete:
            colors.update((v, self.discPalette[colorIndices[self.items[v][colorIndex].value]]) for v in nodes)

        elif colorIndex is not None and self.items.domain[colorIndex].varType == core.VarTypes.String and self.items.domain[colorIndex].name == "label":
            colorIndices = dict((v, i) for i, v in enumerate(set(inst[colorIndex].value for inst in self.items)))
            colors.update((v, self.discPalette[colorIndices[self.items[v][colorIndex].value]]) for v in nodes)
        else:
            colors.update((node, self.discPalette[0]) for node in nodes)

        self.networkCurve.set_node_colors(colors)
        self.replot()

    def set_node_labels(self, attributes=None):
        if self.graph is None:
            return

        nodes = self.graph.nodes()

        if attributes is not None:
            self.node_label_attributes = attributes

        label_attributes = []
        if self.items is not None and isinstance(self.items, data.Table):
            label_attributes = [self.items.domain[att] for att in \
                self.node_label_attributes if att in self.items.domain]

        indices = [[] for u in nodes]
        if self.show_indices:
            indices = [[str(u)] for u in nodes]

        distances = [[] for u in nodes]
        show_distances = False
        if self.label_distances is not None and type(self.label_distances) == \
                           type([]) and len(self.label_distances) == len(nodes):
            distances = self.label_distances
            show_distances = True

        if len(label_attributes) == 0 and \
                        not self.show_indices and not show_distances:
            self.networkCurve.set_node_labels({})

        elif self.trim_label_words > 0:
            self.networkCurve.set_node_labels(dict((node,
                ', '.join(distances[i] + indices[i] +
                          [' '.join(str(self.items[node][att]).split(' ')[:min(self.trim_label_words, len(str(self.items[node][att]).split(' ')))])
                for att in label_attributes])) for i, node in enumerate(nodes)))
        else:
            self.networkCurve.set_node_labels(dict((node, ', '.join(\
                        distances[i] + indices[i] + \
                           [str(self.items[node][att]) for att in \
                           label_attributes])) for i, node in enumerate(nodes)))

        self.replot()


    def set_edge_colors(self, attribute):
        if self.graph is None:
            return

        colorIndices, colorIndex, minValue, maxValue = self.getColorIndeces(self.links, attribute, self.discPalette)
        colors = []

        if colorIndex is not None and self.links.domain[colorIndex].varType == core.VarTypes.Continuous and minValue == maxValue:
            colors = [self.discEdgePalette[0] for edge in self.networkCurve.edge_indices()]

        elif colorIndex is not None and self.links.domain[colorIndex].varType == core.VarTypes.Continuous:
            colors = [self.contPalette[(float(self.links[edge.links_index()][colorIndex].value) - minValue) / (maxValue - minValue)]
                          if str(self.links[edge.links_index()][colorIndex].value) != '?' else
                          self.discPalette[0] for edge in self.networkCurve.edges()]

        elif colorIndex is not None and self.links.domain[colorIndex].varType == core.VarTypes.Discrete:
            colors = [self.discEdgePalette[colorIndices[self.links[edge.links_index()][colorIndex].value]] for edge in self.networkCurve.edges()]

        else:
            colors = [self.discEdgePalette[0] for edge in self.networkCurve.edge_indices()]

        self.networkCurve.set_edge_colors(colors)
        self.replot()

    def set_edge_labels(self, attributes=None):
        if self.graph is None:
            return

        edges = self.networkCurve.edge_indices()

        if attributes is not None:
            self.edge_label_attributes = attributes

        label_attributes = []
        if self.links is not None and isinstance(self.links, data.Table):
            label_attributes = [self.links.domain[att] for att in \
                self.edge_label_attributes if att in self.links.domain]

        weights = [[] for ex in edges]
        if self.show_weights:
            weights = [["%.2f" % self.graph[u][v].get('weight', 1)] for u, v in edges]

        self.networkCurve.set_edge_labels([', '.join(weights[i] + \
                           [str(self.links[i][att]) for att in \
                           label_attributes]) for i, edge in enumerate(edges)])

        self.replot()

    def set_tooltip_attributes(self, attributes):
        if self.graph is None or self.items is None or \
           not isinstance(self.items, data.Table):
            return

        tooltip_attributes = [self.items.domain[att] for att in \
                                 attributes if att in self.items.domain]
        self.networkCurve.set_node_tooltips(dict((node, ', '.join(str( \
                   self.items[node][att]) for att in tooltip_attributes)) \
                                                        for node in self.graph))

    def change_graph(self, newgraph):
        old_nodes = set(self.graph.nodes_iter())
        new_nodes = set(newgraph.nodes_iter())
        inter_nodes = old_nodes & new_nodes
        remove_nodes = list(old_nodes - inter_nodes)
        add_nodes = list(new_nodes - inter_nodes)

        self.graph = newgraph

        if len(remove_nodes) == 0 and len(add_nodes) == 0:
            return False

        current_nodes = self.networkCurve.nodes()

        center_x = numpy.average([node.x() for node in current_nodes.itervalues()]) if len(current_nodes) > 0 else 0
        center_y = numpy.average([node.y() for node in current_nodes.itervalues()]) if len(current_nodes) > 0 else 0

        def closest_nodes_with_pos(nodes):

            neighbors = set()
            for n in nodes:
                neighbors |= set(self.graph.neighbors(n))

            # checked all, none found            
            if len(neighbors - nodes) == 0:
                return []

            inter = old_nodes.intersection(neighbors)
            if len(inter) > 0:
                return inter
            else:
                return closest_nodes_with_pos(neighbors | nodes)

        pos = dict((n, [numpy.average(c) for c in zip(*[(current_nodes[u].x(), current_nodes[u].y()) for u in closest_nodes_with_pos(set([n]))])]) for n in add_nodes)

        self.networkCurve.remove_nodes(remove_nodes)

        nodes = dict((v, self.NodeItem(v, x=pos[v][0] if len(pos[v]) == 2 else center_x, y=pos[v][1] if len(pos[v]) == 2 else center_y, parent=self.networkCurve)) for v in add_nodes)
        self.networkCurve.add_nodes(nodes)
        nodes = self.networkCurve.nodes()

        #add edges
        new_edges = self.graph.edges(add_nodes)

        if self.links is not None and len(self.links) > 0:
            links = self.links
            links_indices = (self.edge_to_row[i + 1][j + 1] for (i, j) in new_edges)

            if self.graph.is_directed():
                edges = [EdgeItem(nodes[i], nodes[j],
                    self.graph[i][j].get('weight', 1), links_index, arrows=EdgeItem.ArrowV, \
                    parent=self.networkCurve) for ((i, j), links_index) in \
                         zip(new_edges, links_indices)]
            else:
                edges = [EdgeItem(nodes[i], nodes[j],
                    self.graph[i][j].get('weight', 1), links_index) for \
                    ((i, j), links_index) in zip(new_edges, \
                                        links_indices, parent=self.networkCurve)]
        elif self.graph.is_directed():
            edges = [EdgeItem(nodes[i], nodes[j], self.graph[i][j].get('weight', 1), \
                    arrows=EdgeItem.ArrowV, parent=self.networkCurve) for (i, j) in new_edges]
        else:
            edges = [EdgeItem(nodes[i], nodes[j], self.graph[i][j].get('weight', 1), \
                    parent=self.networkCurve) for (i, j) in new_edges]

        self.networkCurve.add_edges(edges)

        if len(current_nodes) < 3:
            self.networkCurve.random()

        return True

    def set_graph(self, graph, curve=None, items=None, links=None):
        self.clear()

        if graph is None:
            self.graph = None
            self.networkCurve = None
            self.items = None
            self.links = None
            xMin = -1.0
            xMax = 1.0
            yMin = -1.0
            yMax = 1.0
            self.addMarker("no network", (xMax - xMin) / 2, (yMax - yMin) / 2, alignment=Qt.AlignCenter, size=self.fontSize)
            self.replot()
            return

        self.graph = graph
        self.networkCurve = NetworkCurve() if curve is None else curve()
        self.add_custom_curve(self.networkCurve)

        self.items = items if items is not None else self.graph.items()
        self.links = links if links is not None else self.graph.links()

        #add nodes
        #self.vertices_old = [(None, []) for v in self.graph]
        nodes = dict((v, self.NodeItem(v, parent=self.networkCurve)) for v in self.graph.nodes_iter())
        self.networkCurve.set_nodes(nodes)

        #build edge to row index
        self.edge_to_row = {}
        if self.links is not None and len(self.links) > 0:
            for i, r in enumerate(self.links):
                u = int(r['u'].value)
                v = int(r['v'].value)
                if u - 1 in self.graph and v - 1 in self.graph:
                    u_dict = self.edge_to_row.get(u, {})
                    v_dict = self.edge_to_row.get(v, {})
                    u_dict[v] = i
                    v_dict[u] = i
                    self.edge_to_row[u] = u_dict
                    self.edge_to_row[v] = v_dict
                else:
                    print 'could not find edge', u, v

        #add edges
        if self.links is not None and len(self.links) > 0:
            links = self.links
            links_indices = (self.edge_to_row[i + 1][j + 1] \
                             for (i, j) in self.graph.edges_iter())

            arrow = EdgeItem.ArrowV if self.graph.is_directed() else None

            edges = [EdgeItem(nodes[i], nodes[j],
                    graph[i][j].get('weight', 1), links_index, arrows=arrow, \
                    parent=self.networkCurve) for ((i, j), links_index) in \
                         zip(self.graph.edges_iter(), links_indices)]
        else:
            arrow = EdgeItem.ArrowV if self.graph.is_directed() else None

            edges = [EdgeItem(nodes[i], nodes[j], graph[i][j].get('weight', 1), \
                              arrows=arrow, parent=self.networkCurve) \
                                  for (i, j) in self.graph.edges_iter()]

        self.networkCurve.set_edges(edges)
        self.replot()

    def update_animations(self, use_animations=None):
        OWPlot.update_animations(self, use_animations)
        self.networkCurve.set_use_animations(self.use_animations)

    def set_labels_on_marked(self, labelsOnMarkedOnly):
        self.networkCurve.set_labels_on_marked(labelsOnMarkedOnly)
        self.set_node_labels()
        self.replot()

    def set_show_component_distances(self):
        self.networkCurve.set_show_component_distances(self.show_component_distances)
        self.replot()

    def update_graph_layout(self):
        self._bounds_cache = {}
        self._transform_cache = {}
        OWPlot.update_layout(self)

    def replot(self):
        self.drawComponentKeywords()
        OWPlot.replot(self)



from inspect import *
ofas = formatargspec

def delete_while_accurate(s, accuracy=0.0000001):
    """ Shortens a string containing a float
    while still keeping the desired accuracy. """
    orig = float(s)
    cur = s
    try:
        while abs((float(cur[:-1])-orig)/orig) < accuracy:
            cur = cur[:-1]
    except:
        pass
    return cur

def nfas(*args, **kwargs):

    def fv(value):
        if isinstance(value, float):
            cand = str(value)
            value = delete_while_accurate(cand)
        return "=" + str(value)

    if "formatvalue" not in kwargs and len(args) < 8:
        kwargs["formatvalue"] = fv

    return ofas(*args, **kwargs)

formatargspec = nfas

# inspect.getmembers from Python 2.7
# In Python 2.6 there is try/except missing and function fails on C++-based classes if they define attributes
# Example: AttributeError: 'Plot' object attribute 'animate_points' is an instance attribute
def getmembers27(object, predicate=None):
    """Return all members of an object as (name, value) pairs sorted by name.
    Optionally, only return members that satisfy a given predicate."""
    results = []
    for key in dir(object):
        try:
            value = getattr(object, key)
        except AttributeError:
            continue
        if not predicate or predicate(value):
            results.append((key, value))
    results.sort()
    return results

getmembers = getmembers27

if __name__ == "__main__":
    for a in [ "0", "0.", "-1.3", "1.30000000001" ]:
        print a, "shorter:", delete_while_accurate(a)



"""
.. index:: Network

*********
BaseGraph
*********

BaseGraph provides methods to work with additional data--describing nodes and
edges:

* items (:obj:`Orange.data.Table`) - information on nodes. Each row in the table corresponds to a node with ID matching row index.
* links (:obj:`Orange.data.Table`) - information on edges. Each row in the table corresponds to an edge. Two columns titled "u" and "v" should be specified in the table which contain indices of nodes on the given edge.

The BaseGraph class contains also other methods that are common to the four graph types.
    
.. autoclass:: Orange.network.BaseGraph
   :members:

***********
Graph Types
***********

The reference in this section is complemented with the original NetworkX 
library reference. For a complete documentation please refer to the 
`NetworkX docs <http://networkx.lanl.gov/reference/>`_. All methods from the
NetworkX package can be used for graph analysis and manipulation. For reading
and writing graphs refer to the Orange.network.readwrite docs.

Graph
=====

.. autoclass:: Orange.network.Graph
   :members:

DiGraph
=======
   
.. autoclass:: Orange.network.DiGraph
   :members:

MultiGraph
==========
   
.. autoclass:: Orange.network.MultiGraph
   :members:
   
MultiDiGraph
============
   
.. autoclass:: Orange.network.MultiDiGraph
   :members:
   
"""

import copy
import math

import numpy
import networkx as nx

import Orange
import Orange.core as orangeom

class MdsTypeClass():
    def __init__(self):
        self.componentMDS = 0
        self.exactSimulation = 1
        self.MDS = 2

MdsType = MdsTypeClass()

def _get_doc(doc):
    tmp = doc.replace('nx.', 'Orange.network.')
    return tmp

class BaseGraph():
    """A collection of methods inherited by all graph types (:obj:`Graph`, 
    :obj:`DiGraph`, :obj:`MultiGraph` and :obj:`MultiDiGraph`).
    
    """

    def __init__(self):
        self._items = None
        self._links = None

    def items(self):
        """Return the :obj:`Orange.data.Table` items with data about network 
        nodes.
        
        """

        if self._items is not None and \
                        len(self._items) != self.number_of_nodes():
            print "Warning: items length does not match the number of nodes."

        return self._items

    def set_items(self, items=None):
        """Set the :obj:`Orange.data.Table` items to the given data. Notice 
        that the number of instances must match the number of nodes.
        
        """

        if items is not None:
            if not isinstance(items, Orange.data.Table):
                raise TypeError('items must be of type \'Orange.data.Table\'')
            if len(items) != self.number_of_nodes():
                print "Warning: items length must match the number of nodes."

        self._items = items

    def links(self):
        """Return the :obj:`Orange.data.Table` links with data about network 
        edges.
        
        """

        if self._links is not None \
                    and len(self._links) != self.number_of_edges():
            print "Warning: links length does not match the number of edges."

        return self._links

    def set_links(self, links=None):
        """Set the :obj:`Orange.data.Table` links to the given data. Notice 
        that the number of instances must match the number of edges.
        
        """

        if links is not None:
            if not isinstance(links, Orange.data.Table):
                raise TypeError('links must be of type \'Orange.data.Table\'')
            if len(links) != self.number_of_edges():
                print "Warning: links length must match the number of edges."

        self._links = links

    def to_orange_network(self):
        """Convert the current network to >>Orange<< NetworkX standard. To use
        :obj:`Orange.network` in Orange widgets, set node IDs to be range 
        [0, no_of_nodes - 1].
        
        """

        G = self.__class__()
        node_list = sorted(self.nodes())
        node_to_index = dict(zip(node_list, range(self.number_of_nodes())))
        index_to_node = dict(zip(range(self.number_of_nodes()), node_list))

        G.add_nodes_from(zip(range(self.number_of_nodes()), [copy.deepcopy(self.node[nid]) for nid in node_list]))
        G.add_edges_from(((node_to_index[u], node_to_index[v], copy.deepcopy(self.edge[u][v])) for u, v in self.edges()))

        for id in G.node.keys():
            G.node[id]['old_id'] = index_to_node[id]

        if self.items():
            G.set_items(self.items())

        if self.links():
            G.set_links(self.links())

        return G

    ### TODO: OVERRIDE METHODS THAT CHANGE GRAPH STRUCTURE, add warning prints

    def items_vars(self):
        """Return a list of features in the :obj:`Orange.data.Table` items."""

        vars = []
        if (self._items is not None):
            if isinstance(self._items, Orange.data.Table):
                vars = list(self._items.domain.variables)

                metas = self._items.domain.getmetas(0)
                vars.extend(var for i, var in metas.iteritems())
        return vars

    def links_vars(self):
        """Return a list of features in the :obj:`Orange.data.Table` links."""

        vars = []
        if (self._links is not None):
            if isinstance(self._links, Orange.data.Table):
                vars = list(self._links.domain.variables)

                metas = self._links.domain.getmetas(0)
                vars.extend(var for i, var in metas.iteritems())
        return [x for x in vars if str(x.name) != 'u' and str(x.name) != 'v']

    def subgraph(self, nbunch):
        G = self.__class__.__bases__[1].subgraph(self, nbunch)
        items = self.items().get_items(G.nodes())
        G = G.to_orange_network()
        G.set_items(items)
        return G

class Graph(BaseGraph, nx.Graph):
    """Bases: `NetworkX.Graph <http://networkx.lanl.gov/reference/classes.graph.html>`_, 
    :obj:`Orange.network.BaseGraph` 
    
    """

    def __init__(self, data=None, name='', **attr):
        nx.Graph.__init__(self, data, name=name, **attr)
        BaseGraph.__init__(self)
        # TODO: _links

    __doc__ += _get_doc(nx.Graph.__doc__)
    __init__.__doc__ = _get_doc(nx.Graph.__init__.__doc__)

class DiGraph(BaseGraph, nx.DiGraph):
    """Bases: `NetworkX.DiGraph <http://networkx.lanl.gov/reference/classes.digraph.html>`_, 
    :obj:`Orange.network.BaseGraph` 
    
    """


    def __init__(self, data=None, name='', **attr):
        nx.DiGraph.__init__(self, data, name=name, **attr)
        BaseGraph.__init__(self)

    __doc__ += _get_doc(nx.DiGraph.__doc__)
    __init__.__doc__ = _get_doc(nx.DiGraph.__init__.__doc__)

class MultiGraph(BaseGraph, nx.MultiGraph):
    """Bases: `NetworkX.MultiGraph <http://networkx.lanl.gov/reference/classes.multigraph.html>`_, 
    :obj:`Orange.network.BaseGraph` 
    
    """


    def __init__(self, data=None, name='', **attr):
        nx.MultiGraph.__init__(self, data, name=name, **attr)
        BaseGraph.__init__(self)

    __doc__ += _get_doc(nx.MultiGraph.__doc__)
    __init__.__doc__ = _get_doc(nx.MultiGraph.__init__.__doc__)

class MultiDiGraph(BaseGraph, nx.MultiDiGraph):
    """Bases: `NetworkX.MultiDiGraph <http://networkx.lanl.gov/reference/classes.multidigraph.html>`_, 
    :obj:`Orange.network.BaseGraph` 
    
    """


    def __init__(self, data=None, name='', **attr):
        nx.MultiDiGraph.__init__(self, data, name=name, **attr)
        BaseGraph.__init__(self)

    __doc__ += _get_doc(nx.MultiDiGraph.__doc__)
    __init__.__doc__ = _get_doc(nx.MultiDiGraph.__init__.__doc__)

class GraphLayout(orangeom.GraphLayout):
    """A class for graph layout optimization. Before using any of the layout
    optimization technique, the class have to be initialized with the :obj:`set_graph`
    method. Also, do not forget to call :obj:`set_graph` again if the graph
    structure changes.
    
    .. attribute:: coors
   
        Coordinates of all vertices. They are initialized to random positions.
        You can modify them manually or use one of the optimization algorithms.
        Usage: coors[0][i], coors[1][i]; 0 for x-axis, 1 for y-axis
        
    
    .. automethod:: Orange.network.GraphLayout.set_graph
    
    **Network optimization**
    
    .. automethod:: Orange.network.GraphLayout.random
    
    .. automethod:: Orange.network.GraphLayout.fr
    
    .. automethod:: Orange.network.GraphLayout.fr_radial
    
    .. automethod:: Orange.network.GraphLayout.circular_original
    
    .. automethod:: Orange.network.GraphLayout.circular_random
    
    .. automethod:: Orange.network.GraphLayout.circular_crossing_reduction
    
    **FragViz**
    
    .. automethod:: Orange.network.GraphLayout.mds_components
    
    .. automethod:: Orange.network.GraphLayout.rotate_components
    
    **Helper methods** 
    
    .. automethod:: Orange.network.GraphLayout.get_vertices_in_rect
    
    .. automethod:: Orange.network.GraphLayout.closest_vertex
    
    .. automethod:: Orange.network.GraphLayout.vertex_distances
    
    .. automethod:: Orange.network.GraphLayout.rotate_vertices
    
    **Examples**
    
    *Network constructor and random layout*
    
    In our first example we create a Network object with a simple full graph (K5). 
    Vertices are initially placed randomly. Graph is visualized using pylabs 
    matplotlib. 
        
    :download:`network-constructor-nx.py <code/network-constructor-nx.py>`
    
    .. literalinclude:: code/network-constructor-nx.py
    
    Executing the above saves a pylab window with the following graph drawing:
    
    .. image:: files/network-K5-random.png
    
    *Network layout optimization*
    
    This example demonstrates how to optimize network layout using one of the
    included algorithms.
    
    part of :download:`network-optimization-nx.py <code/network-optimization-nx.py>`
    
    .. literalinclude:: code/network-optimization-nx.py
        :lines: 14-19
        
    The result of the above script is a spring force layout optimization:
    
    .. image:: files/network-K5-fr.png
    
    """

    def __init__(self):
        self.graph = None
        self.items_matrix = None

    def set_graph(self, graph=None, positions=None):
        """Init graph structure.
        
        :param graph: Orange network
        :type graph: Orange.netowork.Graph
        
        :param positions: Initial node positions
        :type positions: A list of positions (x, y)
        
        """
        self.graph = graph

        if positions is not None and len(positions) == graph.number_of_nodes():
            orangeom.GraphLayout.set_graph(self, graph, positions)
        else:
            orangeom.GraphLayout.set_graph(self, graph)

    def random(self):
        """Random graph layout."""

        orangeom.GraphLayout.random(self)

    def fr(self, steps, temperature, coolFactor=0, weighted=False):
        """Fruchterman-Reingold spring layout optimization. Set number of 
        iterations with argument steps, start temperature with temperature 
        (for example: 1000).
        
        """

        return orangeom.GraphLayout.fr(self, steps, temperature, coolFactor, weighted)

    def fr_radial(self, center, steps, temperature):
        """Radial Fruchterman-Reingold spring layout optimization. Set center 
        node with attribute center, number of iterations with argument steps 
        and start temperature with temperature (for example: 1000).
        
        """

        return orangeom.GraphLayout.fr_radial(self, center, steps, temperature)

    def circular_original(self):
        """Circular graph layout with original node order."""

        orangeom.GraphLayout.circular_original(self)

    def circular_random(self):
        """Circular graph layout with random node order."""

        orangeom.GraphLayout.circular_random(self)

    def circular_crossing_reduction(self):
        """Circular graph layout with edge crossing reduction (Michael Baur, 
        Ulrik Brandes).
        
        """

        orangeom.GraphLayout.circular_crossing_reduction(self)

    def get_vertices_in_rect(self, x1, y1, x2, y2):
        """Return a list of nodes in the given rectangle."""

        return orangeom.GraphLayout.get_vertices_in_rect(self, x1, y1, x2, y2)

    def closest_vertex(self, x, y):
        """Return the closest node to given point."""

        return orangeom.GraphLayout.closest_vertex(self, x, y)

    def vertex_distances(self, x, y):
        """Return distances (a list of (distance, vertex) tuples) of all nodes 
        to the given position.
        
        """

        return orangeom.GraphLayout.vertex_distances(self, x, y)

    def rotate_vertices(self, components, phi):
        """Rotate network components for a given angle.
        
        :param components: list of network components
        :type components: list of lists of vertex indices
        
        :param phi: list of component rotation angles (unit: radians)
        :type phi: float
        
        """
        #print phi 
        for i in range(len(components)):
            if phi[i] == 0:
                continue

            component = components[i]

            x = self.coors[0][component]
            y = self.coors[1][component]

            x_center = x.mean()
            y_center = y.mean()

            x = x - x_center
            y = y - y_center

            r = numpy.sqrt(x ** 2 + y ** 2)
            fi = numpy.arctan2(y, x)

            fi += phi[i]
            #fi += factor * M[i] * numpy.pi / 180

            x = r * numpy.cos(fi)
            y = r * numpy.sin(fi)

            self.coors[0][component] = x + x_center
            self.coors[1][component] = y + y_center

    def rotate_components(self, maxSteps=100, minMoment=0.000000001,
                          callbackProgress=None, callbackUpdateCanvas=None):
        """Rotate the network components using a spring model."""

        if self.items_matrix == None:
            return 1

        if self.graph == None:
            return 1

        if self.items_matrix.dim != self.graph.number_of_nodes():
            return 1

        self.stopRotate = 0

        # rotate only components with more than one vertex
        components = [component for component \
            in nx.algorithms.components.connected_components(self.graph) \
            if len(component) > 1]
        vertices = set(range(self.graph.number_of_nodes()))
        step = 0
        M = [1]
        temperature = [[30.0, 1] for i in range(len(components))]
        dirChange = [0] * len(components)
        while step < maxSteps and (max(M) > minMoment or \
                                min(M) < -minMoment) and not self.stopRotate:
            M = [0] * len(components)

            for i in range(len(components)):
                component = components[i]

                outer_vertices = vertices - set(component)

                x = self.coors[0][component]
                y = self.coors[1][component]

                x_center = x.mean()
                y_center = y.mean()

                for j in range(len(component)):
                    u = component[j]

                    for v in outer_vertices:
                        d = self.items_matrix[u, v]
                        u_x = self.coors[0][u]
                        u_y = self.coors[1][u]
                        v_x = self.coors[0][v]
                        v_y = self.coors[1][v]
                        L = [(u_x - v_x), (u_y - v_y)]
                        R = [(u_x - x_center), (u_y - y_center)]
                        e = math.sqrt((v_x - x_center) ** 2 + \
                                      (v_y - y_center) ** 2)

                        M[i] += (1 - d) / (e ** 2) * numpy.cross(R, L)

            tmpM = numpy.array(M)
            #print numpy.min(tmpM), numpy.max(tmpM),numpy.average(tmpM),numpy.min(numpy.abs(tmpM))

            phi = [0] * len(components)
            #print "rotating", temperature, M
            for i in range(len(M)):
                if M[i] > 0:
                    if temperature[i][1] < 0:
                        temperature[i][0] = temperature[i][0] * 5 / 10
                        temperature[i][1] = 1
                        dirChange[i] += 1

                    phi[i] = temperature[i][0] * numpy.pi / 180
                elif M[i] < 0:
                    if temperature[i][1] > 0:
                        temperature[i][0] = temperature[i][0] * 5 / 10
                        temperature[i][1] = -1
                        dirChange[i] += 1

                    phi[i] = -temperature[i][0] * numpy.pi / 180

            # stop rotating when phi is to small to notice the rotation
            if max(phi) < numpy.pi / 1800:
                #print "breaking"
                break

            self.rotate_vertices(components, phi)
            if callbackUpdateCanvas: callbackUpdateCanvas()
            if callbackProgress : callbackProgress(min([dirChange[i] for i \
                                    in range(len(dirChange)) if M[i] != 0]), 9)
            step += 1

    def mds_update_data(self, components, mds, callbackUpdateCanvas):
        """Translate and rotate the network components to computed positions."""

        component_props = []
        x_mds = []
        y_mds = []
        phi = [None] * len(components)
        self.diag_coors = math.sqrt((\
                    min(self.coors[0]) - max(self.coors[0])) ** 2 + \
                    (min(self.coors[1]) - max(self.coors[1])) ** 2)

        if self.mdsType == MdsType.MDS:
            x = [mds.points[u][0] for u in range(self.graph.number_of_nodes())]
            y = [mds.points[u][1] for u in range(self.graph.number_of_nodes())]
            self.coors[0][range(self.graph.number_of_nodes())] = x
            self.coors[1][range(self.graph.number_of_nodes())] = y
            if callbackUpdateCanvas:
                callbackUpdateCanvas()
            return

        for i in range(len(components)):
            component = components[i]

            if len(mds.points) == len(components):  # if average linkage before
                x_avg_mds = mds.points[i][0]
                y_avg_mds = mds.points[i][1]
            else:                                   # if not average linkage before
                x = [mds.points[u][0] for u in component]
                y = [mds.points[u][1] for u in component]

                x_avg_mds = sum(x) / len(x)
                y_avg_mds = sum(y) / len(y)
                # compute rotation angle
                c = [numpy.linalg.norm(numpy.cross(mds.points[u], \
                            [self.coors[0][u], self.coors[1][u]])) \
                            for u in component]
                n = [numpy.vdot([self.coors[0][u], \
                                 self.coors[1][u]], \
                                 [self.coors[0][u], \
                                  self.coors[1][u]]) for u in component]
                phi[i] = sum(c) / sum(n)
                #print phi

            x = self.coors[0][component]
            y = self.coors[1][component]

            x_avg_graph = sum(x) / len(x)
            y_avg_graph = sum(y) / len(y)

            x_mds.append(x_avg_mds)
            y_mds.append(y_avg_mds)

            component_props.append((x_avg_graph, y_avg_graph, \
                                    x_avg_mds, y_avg_mds, phi))

        w = max(self.coors[0]) - min(self.coors[0])
        h = max(self.coors[1]) - min(self.coors[1])
        d = math.sqrt(w ** 2 + h ** 2)
        #d = math.sqrt(w*h)
        e = [math.sqrt((self.coors[0][u] - self.coors[0][v]) ** 2 +
                  (self.coors[1][u] - self.coors[1][v]) ** 2) for
                  (u, v) in self.graph.edges()]

        if self.scalingRatio == 0:
            pass
        elif self.scalingRatio == 1:
            self.mdsScaleRatio = d
        elif self.scalingRatio == 2:
            self.mdsScaleRatio = d / sum(e) * float(len(e))
        elif self.scalingRatio == 3:
            self.mdsScaleRatio = 1 / sum(e) * float(len(e))
        elif self.scalingRatio == 4:
            self.mdsScaleRatio = w * h
        elif self.scalingRatio == 5:
            self.mdsScaleRatio = math.sqrt(w * h)
        elif self.scalingRatio == 6:
            self.mdsScaleRatio = 1
        elif self.scalingRatio == 7:
            e_fr = 0
            e_count = 0
            for i in range(self.graph.number_of_nodes()):
                for j in range(i + 1, self.graph.number_of_nodes()):
                    x1 = self.coors[0][i]
                    y1 = self.coors[1][i]
                    x2 = self.coors[0][j]
                    y2 = self.coors[1][j]
                    e_fr += math.sqrt((x1 - x2) ** 2 + (y1 - y2) ** 2)
                    e_count += 1
            self.mdsScaleRatio = e_fr / e_count
        elif self.scalingRatio == 8:
            e_fr = 0
            e_count = 0
            for i in range(len(components)):
                for j in range(i + 1, len(components)):
                    x_avg_graph_i, y_avg_graph_i, x_avg_mds_i, \
                    y_avg_mds_i, phi_i = component_props[i]
                    x_avg_graph_j, y_avg_graph_j, x_avg_mds_j, \
                    y_avg_mds_j, phi_j = component_props[j]
                    e_fr += math.sqrt((x_avg_graph_i - x_avg_graph_j) ** 2 + \
                                      (y_avg_graph_i - y_avg_graph_j) ** 2)
                    e_count += 1
            self.mdsScaleRatio = e_fr / e_count
        elif self.scalingRatio == 9:
            e_fr = 0
            e_count = 0
            for i in range(len(components)):
                component = components[i]
                x = self.coors[0][component]
                y = self.coors[1][component]
                for i in range(len(x)):
                    for j in range(i + 1, len(y)):
                        x1 = x[i]
                        y1 = y[i]
                        x2 = x[j]
                        y2 = y[j]
                        e_fr += math.sqrt((x1 - x2) ** 2 + (y1 - y2) ** 2)
                        e_count += 1
            self.mdsScaleRatio = e_fr / e_count

        diag_mds = math.sqrt((max(x_mds) - min(x_mds)) ** 2 + (max(y_mds) - \
                                                              min(y_mds)) ** 2)
        e = [math.sqrt((self.coors[0][u] - self.coors[0][v]) ** 2 +
                  (self.coors[1][u] - self.coors[1][v]) ** 2) for
                  (u, v) in self.graph.edges()]
        e = sum(e) / float(len(e))

        x = [mds.points[u][0] for u in range(len(mds.points))]
        y = [mds.points[u][1] for u in range(len(mds.points))]
        w = max(x) - min(x)
        h = max(y) - min(y)
        d = math.sqrt(w ** 2 + h ** 2)

        if len(x) == 1:
            r = 1
        else:
            if self.scalingRatio == 0:
                r = self.mdsScaleRatio / d * e
            elif self.scalingRatio == 1:
                r = self.mdsScaleRatio / d
            elif self.scalingRatio == 2:
                r = self.mdsScaleRatio / d * e
            elif self.scalingRatio == 3:
                r = self.mdsScaleRatio * e
            elif self.scalingRatio == 4:
                r = self.mdsScaleRatio / (w * h)
            elif self.scalingRatio == 5:
                r = self.mdsScaleRatio / math.sqrt(w * h)
            elif self.scalingRatio == 6:
                r = 1 / math.sqrt(self.graph.number_of_nodes())
            elif self.scalingRatio == 7:
                e_mds = 0
                e_count = 0
                for i in range(len(mds.points)):
                    for j in range(i):
                        x1 = mds.points[i][0]
                        y1 = mds.points[i][1]
                        x2 = mds.points[j][0]
                        y2 = mds.points[j][1]
                        e_mds += math.sqrt((x1 - x2) ** 2 + (y1 - y2) ** 2)
                        e_count += 1
                r = self.mdsScaleRatio / e_mds * e_count
            elif self.scalingRatio == 8:
                e_mds = 0
                e_count = 0
                for i in range(len(components)):
                    for j in range(i + 1, len(components)):
                        x_avg_graph_i, y_avg_graph_i, x_avg_mds_i, \
                        y_avg_mds_i, phi_i = component_props[i]
                        x_avg_graph_j, y_avg_graph_j, x_avg_mds_j, \
                        y_avg_mds_j, phi_j = component_props[j]
                        e_mds += math.sqrt((x_avg_mds_i - x_avg_mds_j) ** 2 + \
                                           (y_avg_mds_i - y_avg_mds_j) ** 2)
                        e_count += 1
                r = self.mdsScaleRatio / e_mds * e_count
            elif self.scalingRatio == 9:
                e_mds = 0
                e_count = 0
                for i in range(len(mds.points)):
                    for j in range(i):
                        x1 = mds.points[i][0]
                        y1 = mds.points[i][1]
                        x2 = mds.points[j][0]
                        y2 = mds.points[j][1]
                        e_mds += math.sqrt((x1 - x2) ** 2 + (y1 - y2) ** 2)
                        e_count += 1
                r = self.mdsScaleRatio / e_mds * e_count

            #r = self.mdsScaleRatio / d
            #print "d", d, "r", r
            #r = self.mdsScaleRatio / math.sqrt(self.graph.number_of_nodes())

        for i in range(len(components)):
            component = components[i]
            x_avg_graph, y_avg_graph, x_avg_mds, \
            y_avg_mds, phi = component_props[i]

#            if phi[i]:  # rotate vertices
#                #print "rotate", i, phi[i]
#                r = numpy.array([[numpy.cos(phi[i]), -numpy.sin(phi[i])], [numpy.sin(phi[i]), numpy.cos(phi[i])]])  #rotation matrix
#                c = [x_avg_graph, y_avg_graph]  # center of mass in FR coordinate system
#                v = [numpy.dot(numpy.array([self.coors[0][u], self.coors[1][u]]) - c, r) + c for u in component]
#                self.coors[0][component] = [u[0] for u in v]
#                self.coors[1][component] = [u[1] for u in v]

            # translate vertices
            if not self.rotationOnly:
                self.coors[0][component] = \
                (self.coors[0][component] - x_avg_graph) / r + x_avg_mds
                self.coors[1][component] = \
                (self.coors[1][component] - y_avg_graph) / r + y_avg_mds

        if callbackUpdateCanvas:
            callbackUpdateCanvas()

    def mds_callback(self, a, b=None):
        """Refresh the UI when running  MDS on network components."""

        if not self.mdsStep % self.mdsRefresh:
            self.mds_update_data(self.mdsComponentList,
                               self.mds,
                               self.callbackUpdateCanvas)

            if self.mdsType == MdsType.exactSimulation:
                self.mds.points = [[self.coors[0][i], \
                                    self.coors[1][i]] \
                                    for i in range(len(self.coors))]
                self.mds.freshD = 0

            if self.callbackProgress != None:
                self.callbackProgress(self.mds.avg_stress, self.mdsStep)

        self.mdsStep += 1

        if self.stopMDS:
            return 0
        else:
            return 1

    def mds_components(self, mdsSteps, mdsRefresh, callbackProgress=None, \
                       callbackUpdateCanvas=None, torgerson=0, \
                       minStressDelta=0, avgLinkage=False, rotationOnly=False, \
                       mdsType=MdsType.componentMDS, scalingRatio=0, \
                       mdsFromCurrentPos=0):
        """Position the network components according to similarities among
        them.

        """

        if self.items_matrix == None:
            self.information('Set distance matrix to input signal')
            return 1

        if self.graph == None:
            return 1

        if self.items_matrix.dim != self.graph.number_of_nodes():
            return 1

        self.mdsComponentList = nx.algorithms.components.connected_components(self.graph)
        self.mdsRefresh = mdsRefresh
        self.mdsStep = 0
        self.stopMDS = 0
        self.items_matrix.matrixType = Orange.misc.SymMatrix.Symmetric
        self.diag_coors = math.sqrt((min(self.coors[0]) - \
                                     max(self.coors[0])) ** 2 + \
                                     (min(self.coors[1]) - \
                                      max(self.coors[1])) ** 2)
        self.rotationOnly = rotationOnly
        self.mdsType = mdsType
        self.scalingRatio = scalingRatio

        w = max(self.coors[0]) - min(self.coors[0])
        h = max(self.coors[1]) - min(self.coors[1])
        d = math.sqrt(w ** 2 + h ** 2)
        #d = math.sqrt(w*h)
        e = [math.sqrt((self.coors[0][u] - self.coors[0][v]) ** 2 +
                  (self.coors[1][u] - self.coors[1][v]) ** 2) for
                  (u, v) in self.graph.edges()]
        self.mdsScaleRatio = d / sum(e) * float(len(e))
        #print d / sum(e) * float(len(e))

        if avgLinkage:
            matrix = self.items_matrix.avgLinkage(self.mdsComponentList)
        else:
            matrix = self.items_matrix

        #if self.mds == None: 
        self.mds = Orange.projection.mds.MDS(matrix)

        if mdsFromCurrentPos:
            if avgLinkage:
                for u, c in enumerate(self.mdsComponentList):
                    x = sum(self.coors[0][c]) / len(c)
                    y = sum(self.coors[1][c]) / len(c)
                    self.mds.points[u][0] = x
                    self.mds.points[u][1] = y
            else:
                for u in range(self.graph.number_of_nodes()):
                    self.mds.points[u][0] = self.coors[0][u]
                    self.mds.points[u][1] = self.coors[1][u]

        # set min stress difference between 0.01 and 0.00001
        self.minStressDelta = minStressDelta
        self.callbackUpdateCanvas = callbackUpdateCanvas
        self.callbackProgress = callbackProgress

        if torgerson:
            self.mds.Torgerson()

        self.mds.optimize(mdsSteps, Orange.projection.mds.SgnRelStress, self.minStressDelta, \
                          progress_callback=self.mds_callback)
        self.mds_update_data(self.mdsComponentList, self.mds, callbackUpdateCanvas)

        if callbackProgress != None:
            callbackProgress(self.mds.avg_stress, self.mdsStep)

        del self.rotationOnly
        del self.diag_coors
        del self.mdsRefresh
        del self.mdsStep
        #del self.mds
        del self.mdsComponentList
        del self.minStressDelta
        del self.callbackUpdateCanvas
        del self.callbackProgress
        del self.mdsType
        del self.mdsScaleRatio
        del self.scalingRatio
        return 0

    def mds_components_avg_linkage(self, mdsSteps, mdsRefresh, \
                                   callbackProgress=None, \
                                   callbackUpdateCanvas=None, torgerson=0, \
                                   minStressDelta=0, scalingRatio=0, \
                                   mdsFromCurrentPos=0):
        return self.mds_components(mdsSteps, mdsRefresh, callbackProgress, \
                                   callbackUpdateCanvas, torgerson, \
                                   minStressDelta, True, \
                                   scalingRatio=scalingRatio, \
                                   mdsFromCurrentPos=mdsFromCurrentPos)

    ##########################################################################
    ### BEGIN: DEPRECATED METHODS (TO DELETE IN ORANGE 3.0)                ###
    ##########################################################################

    def map_to_graph(self, graph):
        nodes = sorted(graph.nodes())
        return dict((v, (self.coors[0][i], self.coors[1][i])) for i, v in \
                    enumerate(nodes))


class NxView(object):
    """Network View"""

    def __init__(self, **attr):
        self._network = None
        self._nx_explorer = None

    def set_nx_explorer(self, _nx_explorer):
        self._nx_explorer = _nx_explorer

    def init_network(self, graph):
        return graph

    def node_selection_changed(self):
        pass

    def update_network(self):
        if self._nx_explorer is not None and self._network is not None:
            subnet = self._network
            self._nx_explorer.change_graph(subnet)



"""
<name>SNAP</name>
<description>Read networks from Stanford Large Network Dataset Collection.</description>
<icon>icons/SNAP.svg</icon>
<contact>Miha Stajdohar (miha.stajdohar(@at@)gmail.com)</contact> 
<priority>6415</priority>
"""

import sys
import os.path
import itertools

import PyQt4.QtNetwork

import Orange.data
import Orange.network
import OWGUI

from OWWidget import *

class OWNxSNAP(OWWidget):
    
    settingsList=['last_total']
    
    def __init__(self,parent=None, signalManager = None):
        OWWidget.__init__(self, parent, signalManager, "SNAP")

        self.inputs = []
        self.outputs = [("Network", Orange.network.Graph), ("Items", Orange.data.Table)]
        
        self.last_total = 24763
        
        self.loadSettings()
        
        self.networks = []
        self.tables = []
        
        image = QPixmap(os.path.dirname(__file__) + "/icons/snap_logo.png")

        imageLabel = QLabel(self)
        imageLabel.setPixmap(image)
        self.controlArea.layout().addWidget(imageLabel)
        
        self.controlArea.layout().addStretch(1)
        
        lbl = QLabel("<a href='http://snap.stanford.edu/data/'>http://snap.stanford.edu/data</a>", self)
        lbl.setOpenExternalLinks(True)
        self.controlArea.layout().addWidget(lbl)
        
        scrollArea = QScrollArea(self.mainArea)
        self.mainArea.layout().addWidget(scrollArea)
        
        self.network_list = OWGUI.widgetBox(self.mainArea, addToLayout=False)
        self.network_list.layout().setSizeConstraint(QLayout.SetFixedSize);  
        scrollArea.setWidget(self.network_list);      
        
        self.snap = Orange.network.snap.SNAP()
        self.snap.get_network_list(self.add_tables, self.progress_callback)
        self.progressBarInit()
        self.setMinimumSize(960, 600)
    
    def add_tables(self, networks):
        self.networks = networks
        self.tables = []
        
        if networks is None:
            return
        
        networks.sort(key=lambda net: net.repository)
        for k,g in itertools.groupby(networks, key=lambda net: net.repository):
            network_group = list(g)
        
            if len(network_group) > 0:
                self.network_list.layout().addWidget(QLabel("<h3>" + network_group[0].repository + "</h3>"))
                table = OWGUI.table(self.network_list, rows=len(network_group), columns=5, selectionMode = -1, addToLayout = 1)
                table.setHorizontalHeaderLabels(['Name', 'Type', 'Nodes', 'Edges', 'Description'])
                f = table.font()
                f.setPointSize(9)
                table.setFont(f)
                table.verticalHeader().hide()
                table.setSelectionMode(QAbstractItemView.SingleSelection)
                table.setSelectionBehavior(QAbstractItemView.SelectRows)
                self.connect(table, SIGNAL('itemSelectionChanged()'), lambda table=table: self.select_network(table))
            
                for i, net in enumerate(network_group):
                    lbl = QLabel("<a href='"+ net.link +"'>" + net.name + "</a>")
                    lbl.setOpenExternalLinks(True)
                    table.setCellWidget(i, 0, lbl)
                    OWGUI.tableItem(table, i, 1, net.type)
                    OWGUI.tableItem(table, i, 2, net.nodes)
                    OWGUI.tableItem(table, i, 3, net.edges)
                    OWGUI.tableItem(table, i, 4, net.description)
 
                table.setFixedSize(712, 100)
                table.setColumnWidth(0, 120)
                table.setColumnWidth(1, 80)
                table.setColumnWidth(2, 80)
                table.setColumnWidth(3, 80)
                table.setColumnWidth(4, 350)
                table.resizeRowsToContents()                   
                table.setFixedSize(712, sum(table.rowHeight(i) for i in range(len(networks))) + 27)
                self.tables.append(table)
                
                OWGUI.separator(self.network_list, 10, 10)
        
    def download_progress(self, numblocks, blocksize, filesize):
        try:
            percent = min((numblocks*blocksize*100)/filesize, 100)
            self.progressBarSet(percent)
        except:
            percent = 100
            if numblocks != 0:
                print str(percent)+'%'
        
    def select_network(self, selected_table):
        for table in self.tables:
            selected = table.selectedIndexes()
            if len(selected) > 0:
                row = selected[0].row()
                fn = selected_table.cellWidget(row, 0).text()
                network_info = self.snap.get_network(fn[fn.indexOf('>')+1:-4])
                self.progressBarInit()
                network = network_info.read(progress_callback=self.download_progress)
                self.progressBarFinished()
                self.send('Network', network)
            
            if table is not selected_table:
                table.clearSelection()
        
    
    def progress_callback(self, done, total):
        if done == total:
            self.progressBarFinished()
            return
        
        if total > 0:
            self.progressBarSet(int(done/total * 100))
        else:
            self.progressBarSet(int(done * 100 / self.last_total))
        
if __name__ == "__main__":
    a=QApplication(sys.argv)
    owf=OWNxSNAP()
    owf.activateLoadedSettings()
    owf.show()  
    a.exec_()
    owf.saveSettings()



"""
<name>Net Clustering</name>
<description>Orange widget for community detection in networks</description>
<icon>icons/NetworkClustering.svg</icon>
<contact>Miha Stajdohar (miha.stajdohar(@at@)gmail.com)</contact>
<priority>6430</priority>
"""

import Orange.network
import Orange.network.community as cd
import OWGUI

from OWWidget import *

class OWNxClustering(OWWidget):

    settingsList = ['method', 'iterationHistory', 'autoApply', 'iterations',
                    'hop_attenuation']

    def __init__(self, parent=None, signalManager=None):
        OWWidget.__init__(self, parent, signalManager, 'Nx Clustering')

        self.inputs = [("Network", Orange.network.Graph,
                        self.setNetwork, Default)]
        self.outputs = [("Network", Orange.network.Graph),
                        ("Community Detection", cd.CommunityDetection)]

        self.net = None
        self.method = 0
        self.iterationHistory = 0
        self.autoApply = 0
        self.iterations = 1000
        self.hop_attenuation = 0.1
        self.loadSettings()

        OWGUI.spin(self.controlArea, self, "iterations", 1,
                   100000, 1, label="Iterations: ")
        ribg = OWGUI.radioButtonsInBox(self.controlArea, self, "method",
                                       [], "Method", callback=self.cluster)
        OWGUI.appendRadioButton(ribg, self, "method",
                        "Label propagation clustering (Raghavan et al., 2007)",
                        callback=self.cluster)

        OWGUI.appendRadioButton(ribg, self, "method",
                        "Label propagation clustering (Leung et al., 2009)",
                        callback=self.cluster)
        OWGUI.doubleSpin(OWGUI.indentedBox(ribg), self, "hop_attenuation",
                         0, 1, 0.01, label="Hop attenuation (delta): ")

        self.info = OWGUI.widgetLabel(self.controlArea, ' ')
        OWGUI.checkBox(self.controlArea, self, "iterationHistory",
                       "Append clustering data on each iteration",
                       callback=self.cluster)
        OWGUI.checkBox(self.controlArea, self, "autoApply",
                       "Commit automatically")
        OWGUI.button(self.controlArea, self, "Commit",
                     callback=lambda b=True: self.cluster(b))

        self.cluster()

    def setNetwork(self, net):
        self.net = net
        if self.autoApply:
            self.cluster()

    def cluster(self, btn=False):
        if not btn and not self.autoApply:
            return

        self.info.setText(' ')

        if self.method == 0:
            alg = cd.label_propagation
            kwargs = {'results2items': 1,
                      'resultHistory2items': self.iterationHistory,
                      'iterations': self.iterations}

        elif self.method == 1:
            alg = cd.label_propagation_hop_attenuation
            kwargs = {'results2items': 1,
                      'resultHistory2items': self.iterationHistory,
                      'iterations': self.iterations,
                      'delta': self.hop_attenuation}

        self.send("Community Detection", cd.CommunityDetection(alg, **kwargs))

        if self.net is None:
            self.send("Network", None)
            return

        labels = alg(self.net, **kwargs)

        self.info.setText('%d clusters found' % len(set(labels.values())))
        self.send("Network", self.net)



"""
.. index:: Community Detection in Graphs

.. index::
   single: Network; Community Detection in Graphs

*****************************
Community Detection in Graphs
*****************************

"""

import random
import itertools
import networkx as nx

import Orange.core
import Orange.data


def add_results_to_items(G, lblhistory):
    items = G.items()
    if items is not None and 'clustering label propagation' in items.domain:
        exclude = [attr for attr in items.domain if attr.name == \
                   'clustering label propagation']
        items = Orange.core.Preprocessor_ignore(items, attributes=exclude)

    attrs = [Orange.feature.Discrete('clustering label propagation',
                            values=list(set([l for l in lblhistory[-1]])))]

    dom = Orange.data.Domain(attrs, 0)
    data = Orange.data.Table(dom, [[l] for l in lblhistory[-1]])

    if items is None:
        G.set_items(data)
    else:
        G.set_items(Orange.data.Table([items, data]))


def add_history_to_items(G, lblhistory):
    items = G.items()
    if items is not None:
        exclude = [attr for attr in items.domain if attr.name in \
                   ['c' + str(i) for i in range(1000)]]
        items = Orange.core.Preprocessor_ignore(items, attributes=exclude)

    attrs = [Orange.feature.Discrete('c' + str(i), values=list(set(\
            [l for l in lblhistory[0]]))) for i, _ in enumerate(lblhistory)]

    dom = Orange.data.Domain(attrs, 0)
    # transpose history
    data = map(list, zip(*lblhistory))
    data = Orange.data.Table(dom, data)
    if items is None:
        G.set_items(data)
    else:
        G.set_items(Orange.data.Table([items, data]))


class CommunityDetection(object):

    def __init__(self, algorithm, **kwargs):
        self.algorithm = algorithm
        self.kwargs = kwargs

    def __call__(self, G):
        return self.algorithm(G, **self.kwargs)

def label_propagation_hop_attenuation(G, results2items=0, \
                                      resultHistory2items=0, iterations=1000, \
                                      delta=0.1, node_degree_preference=0):
    """Label propagation for community detection, Leung et al., 2009

    :param G: A Orange graph.
    :type G: Orange.network.Graph

    :param results2items: Append a new feature result to items
        (Orange.data.Table).
    :type results2items: bool

    :param resultHistory2items: Append new features result to items
        (Orange.data.Table) after each iteration of the algorithm.
    :type resultHistory2items: bool

    :param iterations: The max. number of iterations if no convergence.
    :type iterations: int

    :param delta: The hop attenuation factor.
    :type delta: float

    :param node_degree_preference: The power on node degree factor.
    :type node_degree_preference: float

    """

    if G.is_directed():
        raise nx.NetworkXError("""Not allowed for directed graph
              G Use UG=G.to_undirected() to create an undirected graph.""")

    vertices = G.nodes()
    degrees = dict(zip(vertices, [G.degree(v) for v in vertices]))
    labels = dict(zip(vertices, range(G.number_of_nodes())))
    scores = dict(zip(vertices, [1] * G.number_of_nodes()))
    lblhistory = []
    m = node_degree_preference

    for i in range(iterations):
        random.shuffle(vertices)
        stop = 1
        for v in vertices:
            neighbors = G.neighbors(v)
            if len(neighbors) == 0:
                continue

            lbls = sorted(((G.edge[v][u].get('weight', 1), labels[u], u) \
                           for u in neighbors), key=lambda x: x[1])
            lbls = [(sum(scores[u] * degrees[u] ** m * weight for weight, \
                         _u_label, u in group), label) for label, group in \
                    itertools.groupby(lbls, lambda x: x[1])]
            max_score = max(lbls)[0]
            max_lbls = [label for score, label in lbls if score >= max_score]

            # only change label if it is not already one of the
            # preferred (maximal) labels
            if labels[v] not in max_lbls:
                labels[v] = random.choice(max_lbls)
                scores[v] = max(0, max(scores[u] for u in neighbors \
                                       if labels[u] == labels[v]) - delta)
                stop = 0

        lblhistory.append([str(labels[key]) for key in sorted(labels.keys())])
        # if stopping condition is satisfied (no label switched color)
        if stop:
            break

    if results2items and not resultHistory2items:
        add_results_to_items(G, lblhistory)

    if resultHistory2items:
        add_history_to_items(G, lblhistory)

    #print "iterations:", i
    return labels


def label_propagation(G, results2items=0, \
                      resultHistory2items=0, iterations=1000, seed=None):
    """Label propagation for community detection, Raghavan et al., 2007

    :param G: A Orange graph.
    :type G: Orange.network.Graph

    :param results2items: Append a new feature result to items
        (Orange.data.Table).
    :type results2items: bool

    :param resultHistory2items: Append new features result to items
        (Orange.data.Table) after each iteration of the algorithm.
    :type resultHistory2items: bool

    :param iterations: The maximum number of iterations if there is no convergence. 
    :type iterations: int

    """

    if seed is not None:
        random.seed(seed)

    vertices = sorted(G.nodes_iter())
    labels = dict(zip(vertices, range(G.number_of_nodes())))

    def next_label(neighbors):
        """Updating rule as described by Raghavan et al., 2007

        Return a list of possible node labels with equal probability.
        """
        lbls = sorted(labels[u] for u in neighbors)
        lbls = [(len(list(c)), l) for l, c in itertools.groupby(lbls)]
        m = max(lbls)[0]
        return [l for c, l in lbls if c >= m]

    lblhistory = []
    for i in range(iterations):
        random.shuffle(vertices)
        stop = 1
        for v in vertices:
            nbh = G.neighbors(v)
            if len(nbh) == 0:
                continue

            max_lbls = next_label(nbh)

            if labels[v] not in max_lbls:
                stop = 0

            labels[v] = random.choice(max_lbls)

        lblhistory.append([str(labels[key]) for key in sorted(labels.keys())])
        # if stopping condition might be satisfied, check it
        # stop when no label would switch anymore
        if stop:
            for v in vertices:
                nbh = G.neighbors(v)
                if len(nbh) == 0:
                    continue
                max_lbls = next_label(nbh)
                if labels[v] not in max_lbls:
                    stop = 0
                    break

            if stop:
                break

    if results2items and not resultHistory2items:
        add_results_to_items(G, lblhistory)

    if resultHistory2items:
        add_history_to_items(G, lblhistory)

    #print "iterations:", i
    return labels



"""
<name>Net from Distances</name>
<description>Costructs Graph object by connecting nodes from ExampleTable where distance between them is between given threshold.</description>
<icon>icons/NetworkFromDistances.svg</icon>
<contact>Miha Stajdohar (miha.stajdohar(@at@)gmail.com)</contact> 
<priority>6440</priority>
"""

#
# OWNetworkFromDistances.py
#

import copy
import random

import Orange
import OWGUI

from OWNxHist import *
from OWWidget import *
from OWGraph import *
from OWHist import *

class OWNxFromDistances(OWWidget, OWNxHist):
    settingsList=["spinLowerThreshold", "spinUpperThreshold", "netOption", 
                  "dstWeight", "kNN", "percentil", "andor", "excludeLimit"]
    
    def __init__(self, parent=None, signalManager=None):
        OWWidget.__init__(self, parent, signalManager, "Nx from Distances")
        OWNxHist.__init__(self)
        
        self.inputs = [("Distances", Orange.core.SymMatrix, self.setMatrix)]
        self.outputs = [("Network", Orange.network.Graph), 
                        ("Data", Orange.data.Table), 
                        ("Distances", Orange.core.SymMatrix)]

        self.addHistogramControls()
        
        # get settings from the ini file, if they exist
        self.loadSettings()
        
        # GUI
        # general settings
        boxHistogram = OWGUI.widgetBox(self.mainArea, box = "Distance histogram")
        self.histogram = OWHist(self, boxHistogram)
        boxHistogram.layout().addWidget(self.histogram)

        boxHistogram.setMinimumWidth(500)
        boxHistogram.setMinimumHeight(300)
        
        # info
        boxInfo = OWGUI.widgetBox(self.controlArea, box = "Network info")
        self.infoa = OWGUI.widgetLabel(boxInfo, "No data loaded.")
        self.infob = OWGUI.widgetLabel(boxInfo, '')
        self.infoc = OWGUI.widgetLabel(boxInfo, '')
        
        OWGUI.rubber(self.controlArea)
        
        self.resize(700, 100)

    def sendReport(self):
        self.reportSettings("Settings",
                            [("Edge thresholds", "%.5f - %.5f" % \
                              (self.spinLowerThreshold, \
                               self.spinUpperThreshold)),
                             ("Selected vertices", ["All", \
                                "Without isolated vertices", 
                                "Largest component", 
                                "Connected with vertex"][self.netOption]),
                             ("Weight", ["Distance", "1 - Distance"][self.dstWeight])])
        self.reportSection("Histogram")
        self.reportImage(self.histogram.saveToFileDirect, QSize(400,300))
        self.reportSettings("Output graph",
                            [("Vertices", self.matrix.dim),
                             ("Edges", self.nedges),
                             ("Connected vertices", "%i (%.1f%%)" % \
                              (self.pconnected, self.pconnected / \
                               max(1, float(self.matrix.dim))*100))])
        
    def sendSignals(self):
        self.send("Network", self.graph)
        self.send("Distances", self.graph_matrix)    
        if self.graph == None:
            self.send("Data", None)
        else:
            self.send("Data", self.graph.items())
                                                                     
if __name__ == "__main__":    
    appl = QApplication(sys.argv)
    ow = OWNxFromDistances()
    ow.show()
    appl.exec_()



"""
<name>Pubmed Network View</name>
<description></description>
<icon>icons/NetworkExplorer.svg</icon>
<contact></contact> 
<priority>6470</priority>
"""

import Orange
import OWGUI

from OWWidget import *

class PubmedNetworkView(Orange.network.NxView):
    """Network Inside View
    
    """

    def __init__(self, parent):
        Orange.network.NxView.__init__(self)

        self._nhops = 2
        self._edge_threshold = 0.5
        self._algorithm = 0 # 0 without clustering, 1 with clustering
        self._n_max_neighbors = 10
        self._center_nodes = []
        self.parent = parent
        self._hidden_nodes = []
        self._k_algorithm = 0.3
        self._delta_score = {}

    def init_network(self, graph):
        self._network = graph

        if hasattr(self.parent, 'init_network'):
            self.parent.init_network()

        if graph is None:
            return None

        return graph


    def update_network(self):
        if self._center_nodes == []:
            return

        subnet = Orange.network.Graph()
        central_nodes, to_add = self._center_nodes[:], self._center_nodes[:]
        for l in range(self._nhops):
            for i in central_nodes:
                neig = sorted([x for x in self._network.neighbors(i) if self._network.edge[i][x]['weight'] > self._edge_threshold], reverse=True)
                if len(neig) > self._n_max_neighbors:
                    neig = neig[:self._n_max_neighbors]
                to_add.extend(neig)
            central_nodes = neig
        to_add = list(set(to_add))
        subnet.add_nodes_from([(x, self._network.node[x]) for x in to_add])
        nodes = subnet.nodes()
        while nodes:
            i = nodes.pop()
            subnet.node[i] = self._network.node[i]
            neig = [x for x in self._network.neighbors(i) if x in nodes]
            subnet.add_weighted_edges_from([(i, x, w) for x, w in zip(neig, [self._network.edge[i][y]['weight'] for y in neig])])

        subnet.remove_nodes_from(self._hidden_nodes)
        subnet = self._propagate(subnet)

        if self._nx_explorer is not None:
            self._nx_explorer.change_graph(subnet)

    def set_nhops(self, nhops):
        self._nhops = nhops

    def set_edge_threshold(self, edge_threshold):
        self._edge_threshold = edge_threshold

    def set_algorithm(self, algorithm):
        self._algorithm = algorithm

    def set_n_max_neighbors(self, n_max_neighbors):
        self._n_max_neighbors = n_max_neighbors

    def set_center_nodes(self, c_nodes):
        self._center_nodes = c_nodes

    def set_k(self, k):
        self._k_algorithm = k

    def node_selection_changed(self):
        pass

    def set_user_node_score(self, node_index, new_score):
        self._delta_score[node_index] = new_score - self._network.node[node_index]['score']
        self._network.node[node_index]['score'] = new_score
        self._network.node[node_index]['user_score'] = 1

    def update_center_nodes(self, new_node):
        self._center_nodes.append(new_node)

    def _get_neighbors(self):
    #TO DELETE?
        nodes = set([self._center_node])
        for n in range(self._nhops):
            neighbors = set()
            for node in nodes:
                neighbors.update(self._network.neighbors(node))
            nodes.update(neighbors)
        return nodes

    def _propagate(self, net):
        central_nodes = [i for i in net.nodes() if net.node[i]['user_score'] == 1]
        central_delta_score = [self._delta_score[id] for id in central_nodes]
        cluster_centers = [net.node[x]['cluster'] for x in central_nodes if net.node[x]['cluster'] != '']
        updated = central_nodes[:]
        L = 0
        while True:
            to_update = []
            for i in central_nodes:
                to_update.extend(net.neighbors(i))
            to_update = set(to_update) - set(updated)
            if to_update:
                for i in to_update:
                    if not net.node[i]['user_score']:
                        predec = [j for j in net.neighbors(i) if j in central_nodes]
                        predec_delta_score = [central_delta_score[central_nodes.index(x)] for x in predec]
                        self._k_algortihm = self.compute_k(i, cluster_centers, net)
                        if self._k_algortihm: #else the node is isolated and the score doesn't change
                            net.node[i]['score'] = _prop(predec, predec_delta_score, i, L, net)
                        net.node[i]['level'] = L
                updated.extend(to_update)
                central_nodes = list(to_update)
                central_delta_score = []
                for id in central_nodes:
                    if self._delta_score.get(id, []):
                        central_delta_score.append(self._delta_score[id])
                    else:
                        central_delta_score.append(net.node[id]['score'] - 0.5)
                L += 1
            else:
                return net

    def compute_k(self, id, centers, net):
        import networkx as nx #VEDERE SE C'E' IN ORANGENET 
        if self._algorithm == 0: #no clustering
            return self._algorithm
        else:   #clustering
            paths = []
            for c in centers:
                try:
                    paths.append(nx.algorithms.shortest_paths.generic.shortest_path_length(net, id, c))
                except nx.exception.NetworkXNoPath:
                    return None #isolated node

            return float(min(paths)) / 10


    def _prop(self, predec, predec_delta_score, node, L, net):
        edges_weight = [net[i][node]['weight'] for i in predecessors]
        x = [(delta * w + 0.5) / (1 + level * self._k_algorithm) for delta, w in zip(predec_delta_score, edges_weight)]
        return max(x)


class OWPubmedView(OWWidget):

    settingsList = ['_nhops']

    def __init__(self, parent=None, signalManager=None):
        OWWidget.__init__(self, parent, signalManager, 'Pubmed Network View', wantMainArea=0)

        self.inputs = []
        self.outputs = [("Nx View", Orange.network.NxView)]

        self._nhops = 2
        self._edge_threshold = 0.5
        self._n_max_neighbors = 20
        self.selected_titles = []
        self.titles = []
        self.filter = ''
        self.ids = []
        self._selected_nodes = []
        self._algorithm = 0
        self._k_algorithm = 0.3

        self.loadSettings()

        box = OWGUI.widgetBox(self.controlArea, "Paper Selection", orientation="vertical")
        OWGUI.lineEdit(box, self, "filter", callback=self.filter_list, callbackOnType=True)
        self.list_titles = OWGUI.listBox(box, self, "selected_titles", "titles", selectionMode=QListWidget.MultiSelection, callback=self.update_view)
        OWGUI.separator(self.controlArea)
        box_pref = OWGUI.widgetBox(self.controlArea, "Preferences", orientation="vertical")
        OWGUI.spin(box_pref, self, "_nhops", 1, 6, 1, label="Number of hops: ", callback=self.update_view)
        OWGUI.spin(box_pref, self, "_n_max_neighbors", 1, 100, 1, label="Max number of neighbors: ", callback=self.update_view)
        OWGUI.doubleSpin(box_pref, self, "_edge_threshold", 0, 1, step=0.01, label="Edge threshold: ", callback=self.update_view)
        OWGUI.separator(self.controlArea)
        box_alg = OWGUI.widgetBox(self.controlArea, "Interest Propagation Algorithm", orientation="vertical")
        radio_box = OWGUI.radioButtonsInBox(box_alg, self, "_algorithm", [], callback=self.update_view)
        OWGUI.appendRadioButton(radio_box, self, "_algorithm", "Without Clustering", callback=self.update_view)
        OWGUI.doubleSpin(OWGUI.indentedBox(radio_box), self, "_k_algorithm", 0, 1, step=0.01, label="Parameter k: ", callback=self.update_view)
        OWGUI.appendRadioButton(radio_box, self, "_algorithm", "With Clustering", callback=self.update_view)

        self.inside_view = PubmedNetworkView(self)
        self.send("Nx View", self.inside_view)


    def init_network(self):
        if self.inside_view._network is None:
            return

        self.titles = [self.inside_view._network.node[node]['title'] for node in self.inside_view._network.nodes()]
        self.ids = self.inside_view._network.nodes()

        QObject.connect(self.inside_view._nx_explorer.networkCanvas, SIGNAL('point_rightclicked(Point*)'), self.node_menu_show)


    def update_view(self):
        self.inside_view.set_nhops(self._nhops)
        self.inside_view.set_edge_threshold(self._edge_threshold)
        self.inside_view.set_n_max_neighbors(self._n_max_neighbors)
        self.inside_view.set_algorithm(self._algorithm)
        self.inside_view.set_k(self._k_algorithm)
        self._selected_nodes = [self.ids[row] for row in self.selected_titles]
        self.inside_view.set_center_nodes(self._selected_nodes)
        self.inside_view.update_network()


    def filter_list(self):
        """Given a query for similar titles sets titles and ids"""

        str_input = self.filter
        str_input = str_input.strip(' .').lower().split(' ')
        titles2 = [(n, str.lower(self.inside_view._network.node[n]['title'].encode('utf-8').strip(' .')).split(' ')) for n in self.inside_view._network.nodes()] # [(id,title)]
        titles2 = sorted(titles2, key=lambda x: sum(i in str_input for i in x[1]), reverse=True)
        self.titles = [self.inside_view._network.node[x[0]]['title'] for x in titles2]
        self.ids = [x[0] for x in titles2]

    def node_menu_show(self, node):
        menu = QMenu(self)
        menu.addAction('Expand Node')
        menu.addAction('Hide Node')
        qstr = QString('Set Score')
        submenu = menu.addMenu(qstr)
        submenu.addAction('0')
        submenu.addAction('0.2')
        submenu.addAction('0.5')
        submenu.addAction('0.7')
        submenu.addAction('1')
        self.connect(menu, SIGNAL("triggered(QAction*)"), lambda action, node=node: self.node_menu_triggered(action, node))
        menu.popup(QCursor.pos())

    def node_menu_triggered(self, action, node):
        # delete_node_and_neig --> ????        
        if action.text() == 'Expand Node':
            self.inside_view.update_center_nodes(node.index())

        elif action.text() == 'Hide Node':
            self.inside_view._hidden_nodes.append(node.index())

        else:
            self.inside_view.set_user_node_score(node.index(), float(action.text()))

        self.inside_view.update_network()  #chiama propagate




